from .baserun import Baserun
from .unittests_class import BaserunTestCase

init = Baserun.init
test = Baserun.test
log = Baserun.log
log_llm_chat = Baserun.log_llm_chat
log_llm_completion = Baserun.log_llm_completion
flush = Baserun.flush
